// PV227, Lesson 9: Tessellation shaders
#pragma once

#include "../../Framework/PV227.h"

using namespace std;
using namespace PV227;

//---------------------
//----    SCENE    ----
//---------------------

// Shader programs
ShaderProgram nolit_program;
ShaderProgram teapot_program;

// Geometries we use in this lecture
Geometry geom_sphere;
Geometry geom_teapot_patches;

// A camera that allows us to look at the object from different views
SimpleCamera the_camera;

// Textures
GLuint chesterfield_color_tex;
GLuint chesterfield_normal_tex;
GLuint chesterfield_height_tex;

// Data of our materials
MaterialData_UBO WhiteMaterial_ubo;

// Data of the our objects (their position)
ModelData_UBO TeapotModel_ubo;
ModelData_UBO LightModel_ubo;

// Data of all our lights - positions, colors, etc.
PhongLightsData_UBO PhongLights_ubo;

// Data of our camera - view matrix, projection matrix, etc.
CameraData_UBO CameraData_ubo;

// OpenGL query object to get render time of one frame
GLuint RenderTimeQuery;

// Functions that works with scene objects
void reload_shaders();
void init_scene();
void update_scene(int app_time_diff_ms);
void render_scene();

//-------------------
//----    GUI    ----
//-------------------

// Main AntTweakBar object
TwBar *the_gui;

// Variables that are changed with GUI
float render_time_ms;		// Read-only

float light_pos;

bool use_wireframe;

float height_scale;
bool use_normal_mapping;
bool use_displacement_mapping;

bool use_auto_tessellation;
float tessellation_factor;
float triangle_size;

// Callbacks from the GUI
void TW_CALL reload(void *);

// Functions that works with GUI
void init_gui();

//---------------------------
//----    APPLICATION    ----

// Current window size
int win_width = 640;
int win_height = 480;

// Current time of the application in milliseconds, for animations
int app_time_ms = 0;
int last_glut_time = 0;
