#version 430 core

// Input variables
layout (location = 0) in vec4 position;
layout (location = 3) in vec3 tangent;
layout (location = 4) in vec3 bitangent;

// Output variables
out VertexData
{
	vec3 position_ws;	// Position in world space
	vec3 tangent_ws;	// Tangent in world space
	vec3 bitangent_ws;	// Bitangent in world space
} outData;

// Data of the camera
layout (std140, binding = 0) uniform CameraData
{
	mat4 projection;		// Projection matrix
	mat4 projection_inv;	// Inverse of the projection matrix
	mat4 view;				// View matrix
	mat4 view_inv;			// Inverse of the view matrix
	mat3 view_it;			// Inverse of the transpose of the top-left part 3x3 of the view matrix
	vec3 eye_position;		// Position of the eye in world space
};

// Data of the object
layout (std140, binding = 2) uniform ModelData
{
	mat4 model;			// Model matrix
	mat4 model_inv;		// Inverse of the model matrix
	mat3 model_it;		// Inverse of the transpose of the top-left part 3x3 of the model matrix
};

void main()
{
	// Transform the position, tangent, and bitangent into world space.
	// In case of the position, transform it with 4x4 model matrix (as we did in the previous lectures).
	// In case of the tangent and bitangent, transform it with 3x3 model matrix, i.e. use mat3(model).
	outData.position_ws = vec3(model * position);
	outData.tangent_ws = mat3(model) * tangent;
	outData.bitangent_ws = mat3(model) * bitangent;
}